/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to you under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * $Id: Application.java 464373 2006-10-16 04:21:54Z rahul $
 */

package org.apache.shale.examples.sqlbrowser;

import java.util.ArrayList;
import java.util.List;
import javax.faces.FacesException;
import javax.faces.context.FacesContext;
import javax.faces.model.SelectItem;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NameClassPair;
import javax.naming.NamingEnumeration;
import javax.sql.DataSource;
import org.apache.shale.tiger.managed.Bean;
import org.apache.shale.tiger.managed.Scope;

/**
 * <p>Application scope managed bean for the SQL Browser application.</p>
 */
@Bean(name="appbean", scope=Scope.APPLICATION)
public class Application {
    

    public Application() {
        System.err.println("Application constructor()");
    }


    // ------------------------------------------------------ Manifest Constants


    /**
     * <p>The prefix of the JNDI name for all relevant data sources.</p>
     */
    private static final String PREFIX = "java:comp/env";


    // ------------------------------------------------------ Instance Variables


    /**
     * <p>The list of <code>DataSource</code>s available via JNDI for this
     * web application.</p>
     */
    private SelectItem[] dataSources = null;


    // ---------------------------------------------------------- Public Methods


    /**
     * <p>Return the list of <code>DataSource</code>s available via JNDI
     * for this web application.</p>
     */
    public SelectItem[] getDataSources() {

        System.err.println("Application getDataSources()");
        if (dataSources == null) {
            List<SelectItem> list = new ArrayList<SelectItem>();
            internal(list);
            try {
                InitialContext context = new InitialContext();
                append(context, PREFIX, list);
            } catch (Exception e) {
                throw new FacesException(e);
            }
            dataSources = (SelectItem[]) list.toArray(new SelectItem[list.size()]);
        }
        System.err.println("Application getDataSources() --> " + dataSources.length);
        return dataSources;

    }


    // --------------------------------------------------------- Private Methods


    /**
     * <p>Append any data sources found in the specified JNDI context
     * to the specified list.  If a JNDI subcontext is found, apply this
     * method recursively.</p>
     *
     * @param context JNDI context to be searched
     * @param prefix Prefix for the fully qualified resource names to be added
     * @param list List of data sources to which new ones should be added
     */
    private void append(Context context, String prefix, List<SelectItem> list) {

        ClassLoader cl = Thread.currentThread().getContextClassLoader();
        if (cl == null) {
            cl = Application.class.getClassLoader();
        }
        try {
            NamingEnumeration<NameClassPair> entries = context.list(prefix);
            while (entries.hasMore()) {
                NameClassPair entry = entries.next();
                Class clazz = cl.loadClass(entry.getClassName());
                if (DataSource.class.isAssignableFrom(clazz)) {
                    list.add(new SelectItem(prefix + "/" + entry.getName(),
                                            (prefix + "/" + entry.getName()).substring(PREFIX.length())));
                } else if (Context.class.isAssignableFrom(clazz)) {
                    Context search = (Context) context.lookup(entry.getName());
                    append(search, prefix + "/" + entry.getName(), list);
                }
            }
        } catch (Exception e) {
            throw new FacesException(e);
        }

    }


    /**
     * <p>Append the default internal data source for this application.</p>
     *
     * @param list List of <code>SelectItem</code> of available data sources
     */
    private void internal(List<SelectItem> list) {

        FacesContext context = FacesContext.getCurrentInstance();
        // FIXME - localize the label
        list.add(new SelectItem("", "(Internal)"));

    }


}
